package com.uas.ps.inquiry.dao;

import com.uas.ps.entity.Product;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 关于物料的数据库实现方法
 *
 * @author hejq
 * @date 2018-01-13 19:00
 */
@Repository
public interface ProductDao extends JpaSpecificationExecutor<Product>, JpaRepository<Product, Long> {

    /**
     * 通过企业UU号和物料编号查询物料信息
     * <pre>
     *     采用List是如果出现多个不会报错
     * </pre>
     *
     * @param enuu 企业UU号
     * @param code 物料编号
     * @return
     */
    List<Product> findByEnUUAndCode(Long enuu, String code);

    /**
     * 通过型号和品牌查询物料是否存在
     *
     * @param enUU 企业UU
     * @param cmpCode 原厂型号
     * @param brand 品牌
     * @return
     */
    List<Product> findByEnUUAndCmpCodeAndBrand(Long enUU, String cmpCode, String brand);
}
