package com.uas.ps.inquiry.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.Set;

/**
 * 公共询价单主表
 *    <pre>
 *        这里是公共询价转成报价后存的数据信息，与以前的采购询价分开
 *    </pre>
 * @author hejq 2018-01-06
 */
@Table(name = "v$public$inquiry")
@Entity
public class PublicInquiryInfo implements Serializable {

	/**
	 * 序列
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * id
	 */
	@Id
	@Column(name = "in_id")
	private Long id;

	/**
	 * 询价单所属企业UU
	 */
	@Column(name = "in_enuu")
	private Long enUU;

	/**
	 * 询价企业名称（方便搜索）
	 */
	@Column(name = "in_enname")
	private String enName;

	/**
	 * 询价企业信息
	 */
	@OneToOne(cascade = { CascadeType.MERGE, CascadeType.REFRESH })
	@JoinColumn(name = "in_enuu", insertable = false, updatable = false)
	private Enterprise enterprise;

	/**
	 * 询价单所属用户UU
	 */
	@Column(name = "in_recorderuu")
	private Long recorderUU;

	/**
	 * 流水号
	 */
	@Column(name = "in_code")
	private String code;

	/**
	 * 单据归属日期
	 */
	@Column(name = "in_date")
	private Date date;

	/**
	 * 录入人
	 */
	@Column(name = "in_recorder")
	private String recorder;

	/**
	 * 报价截止日期
	 */
	@Column(name = "in_enddate")
	private Date endDate;

	/**
	 * 备注
	 */
	@Column(name = "in_remark")
	private String remark;

	/**
	 * 环保要求
	 */
	@Column(name = "in_environment")
	private String environment;

	/**
	 * 询价明细
	 */
	@OneToMany(mappedBy = "inquiry", cascade = { CascadeType.REFRESH, CascadeType.PERSIST, CascadeType.REMOVE,
			CascadeType.MERGE }, fetch = FetchType.EAGER)
	@OrderBy("number")
	private Set<PublicInquiryItemInfo> inquiryItems;

	/**
	 * 来源系统单据ID
	 */
	@Column(name = "in_sourceid")
	private Long sourceId;

	/**
	 * 是否过期
	 */
	@Column(name = "in_overdue", insertable = false, updatable = false)
	private Short overdue;

	/**
	 * 应用来源，主要是为了平台公共询价做处理
	 */
	@Column(name = "in_sourceapp")
	private String sourceApp;

	/**
	 * 收货地址
	 */
	@Column(name = "in_ship")
	private String ship;

	/**
	 * 是否开票<br>
	 * 1. 是<br>
	 * 0. 否
	 */
	@Column(name = "in_invoice")
	private Short invoice;

	/**
	 * 询价类型
	 */
	@Column(name = "in_inquirytype")
	private String inquirytype;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public String getRecorder() {
		return recorder;
	}

	public void setRecorder(String recorder) {
		this.recorder = recorder;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public Long getEnUU() {
		return enUU;
	}

	public void setEnUU(Long enUU) {
		this.enUU = enUU;
	}

	public String getEnName() {
		return enName;
	}

	public void setEnName(String enName) {
		this.enName = enName;
	}

	public Enterprise getEnterprise() {
        return enterprise;
    }

    public void setEnterprise(Enterprise enterprise) {
        this.enterprise = enterprise;
    }

    public Long getRecorderUU() {
		return recorderUU;
	}

	public void setRecorderUU(Long recorderUU) {
		this.recorderUU = recorderUU;
	}

	public Long getSourceId() {
		return sourceId;
	}

	public void setSourceId(Long sourceId) {
		this.sourceId = sourceId;
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public Set<PublicInquiryItemInfo> getInquiryItems() {
		return inquiryItems;
	}

	public void setInquiryItems(Set<PublicInquiryItemInfo> inquiryItems) {
		this.inquiryItems = inquiryItems;
	}

	public String getEnvironment() {
		return environment;
	}

	public void setEnvironment(String environment) {
		this.environment = environment;
	}

	public Short getOverdue() {
		return overdue;
	}

	public void setOverdue(Short overdue) {
		this.overdue = overdue;
	}

	public String getSourceApp() {
		return sourceApp;
	}

	public void setSourceApp(String sourceApp) {
		this.sourceApp = sourceApp;
	}

	public String getShip() {
		return ship;
	}

	public void setShip(String ship) {
		this.ship = ship;
	}

	public Short getInvoice() {
		return invoice;
	}

	public void setInvoice(Short invoice) {
		this.invoice = invoice;
	}

	public String getInquirytype() {
		return inquirytype;
	}

	public void setInquirytype(String inquirytype) {
		this.inquirytype = inquirytype;
	}
}