package com.uas.ps.inquiry.service;

import com.uas.ps.inquiry.domain.IPage;
import com.uas.ps.inquiry.model.*;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import org.springframework.ui.ModelMap;

import java.util.List;

/**
 * 针对卖家，对询价的操作接口
 *
 * Created by hejq on 2018-01-18.
 */
public interface InquiryForSaleService {

    /**
     * 通过明细查询询价详情
     *
     * @param id 报价明细id
     * @return
     */
    PublicInquiryItem findById(Long id, Long enUU) throws IllegalAccessException;

    /**
     * 供应商报价时上传附件信息
     *
     * @param attach
     * @return
     */
    Attach addAttaches(Attach attach);

    /**
     * 供应商保存报价信息
     *
     * @author hejq
     * @date 2018-01-14 16:43
     * @param item 报价信息
     * @return
     */
    PublicInquiryItem add(PublicInquiryItem item);

    /**
     * 保存询价信息
     *
     * @author hejq
     * @date 2018-01-15 9:16
     * @param items 询价信息
     * @return
     */
    List<PublicInquiryItem> saveItems(List<PublicInquiryItem> items);

    /**
     * 保存询价信息
     *
     * @param inquiryItem 报价信息
     * @return
     */
    PublicInquiryItem save(PublicInquiryItem inquiryItem);

    /**
     * 通过公共询价明细id查询报价信息
     *
     * @param id 公共询价明细id
     * @param enuu 供应商UU
     * @return
     */
    PublicInquiryItem findBySourceIdAndEnuu(Long id, Long enuu);

    /**
     * 转成报价询价单
     *
     * @param item 询价明细
     * @return
     */
    PublicInquiryItem saveItem(PublicInquiryItem item) throws Exception;

    /**
     * 通过id查询公共询价详情
     *
     * @param id 主键id
     * @param enuu 企业uu
     * @return
     */
    ModelMap findByIdAndEnuu(Long id, Long enuu);

    /**
     * 移动端转报价的功能
     *
     * @param id 公共询价明细id
     * @param enuu 报价企业UU号
     * @param replies 报价信息
     * @param leadtime 交货周期
     * @param minOrderQty 最小订购
     * @param minPackQty 最小包装
     * @param useruu 个人UU
     * @param currency 币别
     * @param taxrate 税率
     * @return
     */
    ModelMap quoteInquiry(Long id, Long enuu, Long useruu, List<PublicInquiryReply> replies, Long leadtime, Double minPackQty, Double minOrderQty, String currency, Float taxrate) throws Exception;

    /**
     * 通过分页信息查询询价推荐信息
     *
     * @param pageInfo 分页信息
     * @param searchFilter 过滤条件
     * @return
     */
    IPage<InquiryRemind> findRemindByPageInfo(PageInfo pageInfo, SearchFilter searchFilter);

    /**
     * 通过分页信息查询企业询价推荐信息
     * @param pageInfo
     * @param searchFilter
     * @return
     */
    IPage<InquiryEnRemind> findEnRemindByPageInfo(PageInfo pageInfo, SearchFilter searchFilter);

    /**
     * 分配商机
     * @param userUU
     * @param enRemindId
     * @return
     */
    ModelMap allotEnRemind(Long userUU, Long enRemindId);

    /**
     * 分配平台询价
     * @param userUU
     * @param enUU
     * @param itemId
     * @return
     */
    ModelMap allotInquiry(Long userUU, Long enUU, Long itemId);

    /**
     * 公共询价报价
     *
     * @param publicInquiryItem
     * @return
     */
    ModelMap saveQuote(PublicInquiryItem publicInquiryItem);
}
