package com.uas.ps.inquiry.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.uas.ps.core.util.ContextUtils;
import com.uas.ps.entity.Product;
import com.uas.ps.entity.ProductUsers;
import com.uas.ps.entity.Status;
import com.uas.ps.inquiry.AccessConfiguration;
import com.uas.ps.inquiry.dao.*;
import com.uas.ps.inquiry.domain.IPage;
import com.uas.ps.inquiry.entity.*;
import com.uas.ps.inquiry.exception.NotFoundException;
import com.uas.ps.inquiry.model.*;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import com.uas.ps.inquiry.page.criteria.*;
import com.uas.ps.inquiry.page.exception.IllegalOperatorException;
import com.uas.ps.inquiry.rabbit.MessageInfo;
import com.uas.ps.inquiry.rabbit.RabbitConstants;
import com.uas.ps.inquiry.service.PublicInquiryService;
import com.uas.ps.inquiry.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.support.CorrelationData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.dao.DataAccessException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 公共询价接口实现
 * <p>
 * Created by hejq on 2018-01-13.
 */
@Service
@Transactional
public class PublicInquiryServiceImpl implements PublicInquiryService {

    @Autowired
    private PublicInquiryDao publicInquiryDao;

    @Autowired
    private PurcInquiryDao purcInquiryDao;

    @Autowired
    private ProductDao productDao;

    @Autowired
    private PurcInquiryItemDao purcInquiryItemDao;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private PublicInquiryReplyDao publicInquiryReplyDao;

    @Autowired
    private PurcInquiryItemInfoDao inquiryItemInfoDao;

    @Autowired
    private PublicInquiryItemInfoDao itemInfoDao;

    @Autowired
    private PublicInquiryItemDao publicInquiryItemDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private EnterpriseDao enterpriseDao;

    @Autowired
    private InquiryRemindDao remindDao;

    @Autowired
    private ProductReplaceDao productReplaceDao;

    @Autowired
    private InquiryEnRemindDao inquiryEnRemindDao;

    @Autowired
    private KindConcernDao kindConcernDao;

    @Autowired
    private InquiryDataDao inquiryDataDao;

    @Autowired
    private RabbitTemplate rabbitTemplate;
    /**
     * 应用来源
     */
    private final String SOURCERAPP = "B2B";

    /**
     * 接收应用(以","拼接应用名称)
     */
    private final String CONSUMERAPP = "MALL,B2B";

    /**
     * 消息类型 （在B2B消息中拼接跳转单据详情url请求时需要）
     */
    private final String INQUIRY_TYPE = "公共询价";

    /**
     * 消息类型 （在B2B消息中拼接跳转单据详情url请求时需要）
     */
    private final String INQUIRY_TYPE_MALL = "MALL跳转买家已报价页面";

    /**
     * 消息类型 （在MALL消息中拼接求购询价待报价）
     */
    private final String INQUIRY_TYPE_SELLER_MALL = "MALL公共询价";

    /**
     * 消息类型，公共询价的采纳结果
     */
    private final String INQUIRY_DECIDE_TYPE = "公共询价采纳结果";

    /**
     * 消息类型，公共询价的采纳结果
     */
    private final String INQUIRY_DECIDE_TYPE_MALL = "商城公共询价采纳结果";

    /**
     * 询价种类
     */
    private final String INQUIRYKIND = "publicInquiry";

    /**
     * 消息推送邮件模板id
     */
    private final String MAIL_TEMP_ID = "d94e931b-b699-46d7-8bb2-94604a3352c0";

    /**
     * 消息推送短信模板id
     */
    private final String SMS_TEMP_ID = "6e094dcf-f2c7-462a-9686-b2d32d684a78";

    /**
     * 消息推送方式：邮件、短信、im
     */
    private final String SMS_TYPE = "MAIL_AND_SM_AND_IM";

    /**
     * 来源应用，ERP
     */
    private final String PRODUCERAPP = "ERP";

    /**
     * 消费类型： 多个，MULTI
     */
    private final String CUST_TYPE = "MULTI";


    /**
     * 公共物料访问地址
     */
    private final String PS_PRODUCT_URL = ContextUtils.getBean(AccessConfiguration.class).getPsProductUrl();

    /**
     * 商城链接
     */
    private final String USOFTMALL_URL = ContextUtils.getBean(AccessConfiguration.class).getMallUrl();

    /**
     * 日志
     */
    private static final Logger log = LoggerFactory.getLogger(PublicInquiryServiceImpl.class);

    /**
     * 保存公共询价信息
     *
     * @param inquiries erp传入公共询价的信息
     * @param enuu      询价企业UU号
     * @param address   询价企业地址
     */
    @Override
    public void save(List<BatchInquiry> inquiries, Long enuu, String address) throws NotFoundException {
        List<PurcInquiry> purcInquiries = covert(inquiries, enuu, address);
        if (!CollectionUtils.isEmpty(purcInquiries)) {
            for (PurcInquiry inquiry : purcInquiries) {
                // 判断单号是否已存在
                List<PurcInquiry> old = purcInquiryDao.findByCodeAndEnUU(inquiry.getCode(), enuu);
                List<PurcInquiryItem> items = new ArrayList<PurcInquiryItem>();
                if (CollectionUtils.isEmpty(old) && !CollectionUtils.isEmpty(inquiry.getInquiryItems())) {
                    for (PurcInquiryItem item : inquiry.getInquiryItems()) {
                        item.setInquiry(inquiry);
                        item.setValid(1);
                        items.add(item);
                    }
                }
                try {
                    items = purcInquiryItemDao.save(items);
                    if (!CollectionUtils.isEmpty(items)) {
                        final List<PurcInquiryItem> purcInquiryItems = items;
                        ThreadUtils.task(new Runnable() {
                            @Override
                            public void run() {
                                try {
                                    notifyMessage(purcInquiryItems, PRODUCERAPP);
                                } catch (Exception e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 消息推送
     *
     * @param inquiryItems 询价明细
     * @param sourceApp    消息来源应用
     */
    @Override
    public void notifyMessage(List<PurcInquiryItem> inquiryItems, String sourceApp) throws Exception {
        for (PurcInquiryItem item : inquiryItems) {
            String cmpCode = null;
            if (item.getCmpCode() != null) {
                cmpCode = item.getCmpCode();
            } else if (null != item.getProductId()) {
                Product product = productDao.findOne(item.getProductId());
                cmpCode = product.getCmpCode();
            }
            if (null != cmpCode) {
                String url = PS_PRODUCT_URL + "/productuser/getProductUsersByEnUUAndCmpCode";
                JSONObject formData = new JSONObject();
                formData.put("enUU", item.getInquiry().getEnUU());
                formData.put("cmpCode", cmpCode);
                try {
                    long start = System.currentTimeMillis();
                    log.info("根据企业UU和原厂型号获取个人库记录，enUU:{}, cmpCode:{}", item.getInquiry().getEnUU(), cmpCode);
                    HttpUtil.Response res = HttpUtil.sendGetRequest(url, formData);
                    log.info("/productuser/getProductUsersByEnUUAndCmpCode,耗时：{}", (System.currentTimeMillis() - start));
                    if (res.getStatusCode() == 200) {
                        List<ProductUsers> users = FlexJsonUtils.fromJsonArray(res.getResponseText(), ProductUsers.class);
                        users = getProductReplaceByEnUUAndCmpCode(item.getInquiry().getRecorderUU(), cmpCode, users);
                        if (!CollectionUtils.isEmpty(users)) {
                            List<MessageModel> models = new ArrayList<MessageModel>();
                            User buyerUser = userDao.findOne(null != item.getInquiry().getRecorderUU() ? item.getInquiry().getRecorderUU() : 0L);
                            Enterprise buyerEnterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
                            for (ProductUsers user : users) {
                                String cmp = cmpCode.length() > 10 ? cmpCode.substring(0, 10).concat("...") : cmpCode;
                                String content = "发布关于“型号：" + cmp + "”的公共询价信息，邀请您参与报价。";
                                MessageModel model = new MessageModel();

                                if (null != buyerEnterprise) {
                                    content = buyerEnterprise.getEnName() + content;
                                    model.setBuyerName(buyerEnterprise.getEnName());
                                } else if (null != buyerUser) {
                                    content = buyerUser.getUserName() + content;
                                    model.setBuyerName(buyerUser.getUserName());
                                }
                                // 询价方信息
                                if (null != item.getUserUU() || null != item.getInquiry().getRecorderUU()) {
                                    model.setSenderUu(item.getUserUU() != null ? item.getUserUU() : item.getInquiry().getRecorderUU());
                                } else {
                                    model.setSenderUu(null != buyerEnterprise ? buyerEnterprise.getAdminUU() : 0L);
                                }
                                model.setSenderEnuu(item.getInquiry().getEnUU());
                                model.setCmpCode(cmpCode);
                                model.setType(INQUIRY_TYPE);
                                model.setType(INQUIRY_TYPE_SELLER_MALL);
                                model.setProducerApp(sourceApp);
                                model.setConsumerType(CUST_TYPE);
                                model.setConsumerApp(CONSUMERAPP);
                                model.setContent(content);
                                model.setRemark(String.valueOf(item.getInquiry().getId()));
                                model.setSourceId(item.getId());
                                // 供应商信息
                                Enterprise vendEnterprise = enterpriseDao.findOne(user.getEnuu());
                                if (vendEnterprise != null) {
                                    model.setVendName(vendEnterprise.getEnName());
                                }
                                model.setReceiverEnuu(user.getEnuu());
                                model.setReceiverUu(user.getUseruu());
                                model.setSmsType("DONT_SEND");
                                model.setMailTemplate(MAIL_TEMP_ID);
                                model.setSmTemplate(SMS_TEMP_ID);
                                models.add(model);
                            }
                            if (!CollectionUtils.isEmpty(models)) {
                                sendMessage(models);
                                sendWechatMessage(models);
                            }
                        }
                        // 生成推荐信息
                        remindInquiry(item, users);
                    }
                } catch (Exception e) {
                    String reason = e.getMessage().length() > 999 ? e.getMessage().substring(0, 999).concat("") : e.getMessage();
                    log.info("公共询价产生消息,根据物料型号【{}】查询接收人失败,id:{},原因：{}, Constant:{}, 报价的应用来源:{}", cmpCode, item.getId(), reason, Constant.UU, InquirySource.ERP.name());
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 微信消息发送
     * @param models
     */
    private void sendWechatMessage(final List<MessageModel> models) {
        ThreadUtils.task(new Runnable() {
            @Override
            public void run() {
                try {
                    long start = System.currentTimeMillis();
                    log.info("发送微信消息");
                    String res = HttpUtil.doPost(USOFTMALL_URL + "/wx/sendTemplateMessage", FlexJsonUtils.toJsonDeep(models));
                    log.info("微信消息请求发送日志, {}", res);
                    log.info("微信消息请求发送,数量：{}, 耗时：{}", models.size(), (System.currentTimeMillis() - start));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 根据企业UU和物料原厂型号cmpCode获取替代物料信息
     *
     * @param cmpCode 物料原厂型号
     * @return
     */
    public List<ProductUsers> getProductReplaceByEnUUAndCmpCode(final Long userUU, final String cmpCode, List<ProductUsers> productUsers) {
        List<ProductReplace> productReplacesList = productReplaceDao.findAll(new Specification<ProductReplace>() {
            @Override
            public Predicate toPredicate(Root<ProductReplace> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                final PredicateFactory<ProductReplace> factory = new PredicateFactory<>();
                factory.add(PredicateUtils.ne("userUU", userUU, false));
                factory.add(PredicateUtils.and(PredicateUtils.isNotNull("ptrCmpcode"), PredicateUtils.eq("ptrCmpcode", cmpCode, false)));
                query.where(factory.toPredicate(root, query, cb));
                return null;
            }
        });
        if (!CollectionUtils.isEmpty(productReplacesList)) {
            for (ProductReplace productReplace : productReplacesList) {
                ProductUsers user = new ProductUsers();
                user.setEnuu(productReplace.getEnuu());
                user.setUseruu(productReplace.getUserUU());
                productUsers.add(user);
            }
        }
        // 去掉同一企业下同一个人相同原厂型号的不同物料的记录，保证一个原厂型号只发一条
        List<ProductUsers> returnList = new ArrayList<>();
        Set<String> enUUAndUserUUs = new HashSet<>();
        for (ProductUsers user : productUsers) {
            String receiverInfo = user.getEnuu().toString() + user.getUseruu().toString();
            if (!enUUAndUserUUs.contains(receiverInfo)) {
                returnList.add(user);
                enUUAndUserUUs.add(receiverInfo);
            }
        }
        return returnList;
    }

    /**
     * 通过查询的个人物料相关联的信息，自动生成推荐的报价的数据
     *
     * @param item  报价明细
     * @param users 用户信息
     */
    private void remindInquiry(final PurcInquiryItem item, final List<ProductUsers> users) {
        ThreadUtils.task(new Runnable() {
            @Override
            public void run() {
                try {
                    Map<Long, ProductUsers> productUsersMap = new HashMap<>();
                    Enterprise enterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
                    if (!CollectionUtils.isEmpty(users)) {
                        List<InquiryRemind> inquiryReminds = new ArrayList<>();
                        for (ProductUsers u : users) {
                            List<InquiryRemind> remindList = remindDao.findByVendUserUUAndVendUUAndItemId(u.getUseruu(), u.getEnuu(), item.getId());
                            if (remindList.size() == 0 && item.getInquiry().getEnUU() != null) {
                                productUsersMap.put(u.getEnuu(), u);
                                InquiryRemind remind = new InquiryRemind();
                                remind.setEnName(enterprise.getEnName());
                                remind.setEnUU(item.getInquiry().getEnUU());
                                remind.setUserName(item.getUserName());
                                remind.setUserTel(item.getUserTel());
                                remind.setCode(item.getInquiry().getCode());
                                remind.setBrand(item.getInbrand());
                                remind.setInbrand(item.getInbrand());
                                remind.setCmpCode(item.getCmpCode());
                                remind.setSpec(item.getSpec());
                                remind.setProdTitle(item.getProdTitle());
                                remind.setInid(item.getInquiry().getId());
                                if (item.getProduct() != null) {
                                    remind.setBrand(item.getProduct().getBrand());
                                    remind.setCmpCode(item.getProduct().getCmpCode());
                                    remind.setSpec(item.getProduct().getSpec());
                                    remind.setProdTitle(item.getProduct().getTitle());
                                }
                                remind.setItemId(item.getId());
                                remind.setNeedQty(item.getNeedquantity());
                                remind.setReDate(item.getInquiry().getDate());
                                remind.setEndDate(item.getInquiry().getEndDate());
                                remind.setVendUU(u.getEnuu());
                                remind.setVendUserUU(u.getUseruu());
                                remind.setDate(new Date(System.currentTimeMillis()));
                                remind.setStatus(Status.INPUTTING.value());
                                inquiryReminds.add(remind);
                            }
                        }
                        if (!CollectionUtils.isEmpty(inquiryReminds)) {
                            remindDao.save(inquiryReminds);
                        }
                    }

                    // 查询已关注类目中是否有当前类目
                    String kind = item.getProdTitle();
                    Set<Long> enuuSet = new HashSet<>();
                    if (!StringUtils.isEmpty(kind)) {
                        enuuSet = kindConcernDao.findByNameCnAndNeEnuu(kind, item.getInquiry().getEnUU());
                    }
                    String url = PS_PRODUCT_URL + "/product/businessOpportunity";
                    JSONObject formData = new JSONObject();
                    formData.put("enUU", item.getInquiry().getEnUU());
                    formData.put("pCmpCode", item.getCmpCode());
                    long start = System.currentTimeMillis();
                    log.info("enUU和cmpCode获取其他有相同物料的企业号, enUU:{}, code:{}", item.getInquiry().getEnUU(), item.getCmpCode());
                    HttpUtil.Response res = HttpUtil.sendGetRequest(url, formData);
                    log.info("/product/businessOpportunity , 型号：{}, 耗时：{}", item.getCmpCode(), (System.currentTimeMillis() - start));
                    if (res.getStatusCode() == HttpStatus.OK.value()) {
                        Map<String, Object> result = FlexJsonUtils.fromJson(res.getResponseText());
                        if (!StringUtils.isEmpty(result.get("enUUs"))) {
                            List<Long> enuus = (List<Long>) result.get("enUUs");
                            enuuSet.addAll(enuus);
                        }
                        if (!CollectionUtils.isEmpty(enuuSet)) {
                            List<InquiryEnRemind> inquiryEnReminds = new ArrayList<>();
                            for (Long enuu : enuuSet) {
                                InquiryEnRemind remind = new InquiryEnRemind();
                                remind.setEnName(enterprise.getEnName());
                                remind.setEnUU(item.getInquiry().getEnUU());
                                remind.setUserName(item.getUserName());
                                remind.setUserTel(item.getUserTel());
                                remind.setCode(item.getInquiry().getCode());
                                remind.setBrand(item.getInbrand());
                                remind.setInbrand(item.getInbrand());
                                remind.setCmpCode(item.getCmpCode());
                                remind.setSpec(item.getSpec());
                                remind.setProdTitle(item.getProdTitle());
                                remind.setInid(item.getInquiry().getId());
                                if (item.getProduct() != null) {
                                    remind.setBrand(item.getProduct().getBrand());
                                    remind.setCmpCode(item.getProduct().getCmpCode());
                                    remind.setSpec(item.getProduct().getSpec());
                                    remind.setProdTitle(item.getProduct().getTitle());
                                }
                                remind.setItemId(item.getId());
                                remind.setNeedQty(item.getNeedquantity());
                                remind.setReDate(item.getInquiry().getDate());
                                remind.setEndDate(item.getInquiry().getEndDate());
                                remind.setVendUU(enuu);
                                if (productUsersMap.containsKey(enuu)) {
                                    remind.setRemindStatus(1);
                                } else {
                                    remind.setRemindStatus(0);
                                }
                                remind.setDate(new Date(System.currentTimeMillis()));
                                remind.setStatus(Status.INPUTTING.value());
                                inquiryEnReminds.add(remind);
                            }
                            if (!CollectionUtils.isEmpty(inquiryEnReminds)) {
                                inquiryEnRemindDao.save(inquiryEnReminds);
                            }
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 通过id查询询价信息
     *
     * @param id 公共询价主表id
     * @return
     */
    @Override
    public PurcInquiry findInquiryById(Long id) {
        PurcInquiry inquiry = purcInquiryDao.findOne(id);
        if (!CollectionUtils.isEmpty(inquiry.getInquiryItems())) {
            for (PurcInquiryItem item : inquiry.getInquiryItems()) {
                if (null == item.getProductId()) {
                    Product product = new Product();
                    product.setTitle(item.getProdTitle());
                    product.setCmpCode(item.getCmpCode());
                    product.setBrand(item.getInbrand());
                    product.setSpec(item.getSpec());
                    item.setProduct(product);
                }
            }
        }
        return inquiry;
    }

    /**
     * 发送消息推送
     *
     * @param models 封装的消息信息
     */
    private void sendMessage(final List<MessageModel> models) {
        ThreadUtils.task(new Runnable() {
            @Override
            public void run() {
                try {
                    long start = System.currentTimeMillis();
                    log.info("发送消息");

                    //设置消息ID
                    MessageInfo messageInfo = new MessageInfo();
                    messageInfo.setMsgId(UUID.randomUUID().toString());
                    //设置时间戳
                    messageInfo.setTimestamp(System.currentTimeMillis());
                    messageInfo.setData(models);
                    CorrelationData correlationData = new CorrelationData(messageInfo.getMsgId());
                    rabbitTemplate.convertAndSend(RabbitConstants.AMQ_DIRECT, RabbitConstants.INQUIRY_MESSAGE_ROUTINGKEY, messageInfo, correlationData);

                    log.info("消息请求发送");
                    log.info("/messages , 条数：{},耗时：{}", models.size(), (System.currentTimeMillis() - start));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /**
     * 将ERP传入的inquiry信息转成公共询价服务中心需要的信息
     *
     * @param inquiries 将ERP传入的inquiry信息
     * @param enuu      询价企业UU号
     * @param address   询价企业地址
     * @return
     */
    @Override
    public List<PurcInquiry> covert(List<BatchInquiry> inquiries, Long enuu, String address) throws NotFoundException {
        List<PurcInquiry> purcInquiries = new ArrayList<PurcInquiry>();
        if (!CollectionUtils.isEmpty(inquiries)) {
            for (BatchInquiry inquiry : inquiries) {
                List<PurcInquiry> old = purcInquiryDao.findByCodeAndEnUU(inquiry.getBi_code(), enuu);
                //先判断是否已经存在,不存在才进行存储
                if (CollectionUtils.isEmpty(old)) {
                    PurcInquiry purcInquiry = inquiry.covert();
                    purcInquiry.setEnUU(enuu);
                    purcInquiry.setShip(address);
                    Enterprise e = enterpriseDao.findOne(enuu);
                    if (null != e) {
                        purcInquiry.setEnName(e.getEnName());
                    }
                    Set<PurcInquiryItem> purcInquiryItems = new HashSet<PurcInquiryItem>();
                    if (!CollectionUtils.isEmpty(inquiry.getInProducts())) {
                        for (BatchInProduct batch : inquiry.getInProducts()) {
                            PurcInquiryItem item = new PurcInquiryItem();
                            List<Product> products = productDao.findByEnUUAndCode(enuu, batch.getBip_prodcode());
                            if (!CollectionUtils.isEmpty(products)) {
                                item.setProductId(products.get(0).getId());
                                item.setCmpCode(products.get(0).getCmpCode());
                                item.setInbrand(products.get(0).getBrand());
                                item.setProdTitle(products.get(0).getTitle());
                                item.setSpec(products.get(0).getSpec());
                            } else {
                                item.setCmpCode(batch.getPr_orispeccode());
                                item.setInbrand(batch.getPr_brand());
                                item.setProdTitle(batch.getPr_detail());
                                item.setSpec(batch.getPr_spec());
                            }
                            item.setProdCode(batch.getBip_prodcode());
                            item.setNumber(batch.getBip_detno());
                            item.setFromDate(new Date(System.currentTimeMillis()));
                            item.setStatus((short) Status.NOT_REPLY.value());
                            item.setSource("ERP");
                            item.setSourceid(batch.getBip_id());
                            item.setCurrency(batch.getBip_currency());
                            item.setUserName(batch.getEm_name());
                            item.setUserTel(batch.getBip_mobile());
                            item.setOfferAmount(0);
                            item.setEndDate(inquiry.getBi_enddate());
                            item.setNeedquantity(batch.getYcgqty());
                            item.setEnuu(enuu);
                            if (null == item.getCmpCode() || item.getCmpCode().equals("无")) {
                                item.setCmpCode(item.getSpec());
                            }
                            if (null != inquiry.getBi_date()) {
                                item.setDate(inquiry.getBi_date());
                            } else {
                                item.setDate(new Date(System.currentTimeMillis()));
                            }
                            purcInquiryItems.add(item);
                        }
                    }
                    purcInquiry.setInquiryItems(purcInquiryItems);
                    purcInquiries.add(purcInquiry);
                }
            }
        }
        return purcInquiries;
    }

    /**
     * erp提交审核时，公共询价服务中心单据禁用报价功能
     *
     * @param inquiries erp询价信息
     * @param enuu      询价企业UU号
     */
    @Override
    public void updateStatus(List<BatchInquiry> inquiries, Long enuu) {
        if (!CollectionUtils.isEmpty(inquiries)) {
            for (BatchInquiry inquiry : inquiries) {
                List<PurcInquiry> purcInquiryList = purcInquiryDao.findByCodeAndEnUU(inquiry.getBi_code(), enuu);
                for (PurcInquiry purcInquiry : purcInquiryList) {
                    purcInquiryItemDao.updateStatus(purcInquiry.getId());
                }
            }
        }
    }

    /**
     * 查询供应商的报价信息
     *
     * @return
     */
    @Override
    public List<InquiryDetail> findNotUploadReply(Long enuu) {
        List<PublicInquiryItem> items = publicInquiryItemDao.findByEnUUAndBackStatus(enuu, (short) Status.NOT_UPLOAD.value());
        if (!CollectionUtils.isEmpty(items)) {
            // 防止下载过程中进行操作，这里设置一个下载中的状态
            Set<Long> ids = new HashSet<>();
            for (PublicInquiryItem item : items) {
                ids.add(item.getId());
            }
            if (ids.size() > 0) {
                publicInquiryItemDao.updateBackStatus(ids);
            }
            return convertPublicInquiryReply(items);
        }
        return new ArrayList<>();
    }

    /**
     * 查找报价的单据，上传到卖家UAS
     *
     * @param enUU 企业UU
     * @return
     */
    @Override
    public List<PublicInquiryItem> findNotUploadToUas(Long enUU) {
        return publicInquiryItemDao.findByVendUUAndReplySendStatus(enUU, (short) Status.NOT_UPLOAD.value());
    }

    /**
     * UAS获取商城账套下企业商机
     *
     * @param vendUU 报价企业UU
     * @return 待下载的企业商机
     * @author dongbw
     */
    @Override
    public List<InquiryEnRemind> findNotUploadEnRemind(Long vendUU) {
        return inquiryEnRemindDao.findNotUploadEnRemind(vendUU);
    }

    /**
     * UAS获取商城账套下企业商机之后 回调 修改状态
     *
     * @param idArray 报价单明细id list
     * @param vendUU    报价企业UU
     */
    @Override
    public void downEnRemindSuccess(String[] idArray, Long vendUU) {
        List<InquiryEnRemind> items = new ArrayList<>();
        for (String id : idArray) {
            InquiryEnRemind item = inquiryEnRemindDao.findOne(Long.parseLong(id));
            if (item != null && vendUU.equals(item.getVendUU())) {
                item.setErpDownStatus((short) Status.DOWNLOADED.value());
                items.add(item);
            }
        }
        if (!CollectionUtils.isEmpty(items)) {
            inquiryEnRemindDao.save(items);
        }
    }

    /**
     * 获取当前月和上个月询价单数量
     *
     * @return 数量map
     */
    @Override
    public ModelMap getCountOfLastAndThisMonth() {
        Calendar now = Calendar.getInstance();
        Integer year = now.get(Calendar.YEAR);
        Integer month = now.get(Calendar.MONTH) + 1;
        ModelMap result = new ModelMap();
        try {
            long start = System.currentTimeMillis();
            Long current = inquiryItemInfoDao.countByMonth(year, month);
            log.info("查询本月询价单耗时:{}", (System.currentTimeMillis() - start));
            start = System.currentTimeMillis();
            result.put("current", current);
            if (month == 1) {
                year = year - 1;
                month = 12;
            } else {
                month = month - 1;
            }
            Long last = inquiryItemInfoDao.countByMonth(year, month);
            log.info("查询本月询价单耗时:{}", (System.currentTimeMillis() - start));
            result.put("last", last);
            result.put("success", "true");
        } catch (Exception e) {
            result.put("message", e.getMessage());
            result.put("success", false);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 查询单条bom询价单的报价情况
     *                      增加价格分析 dongbw  2018年9月5日 14:19:07
     * @param inquiry 询价单
     * @return 询价单
     */
    @Override
    public PurcInquiry getQuotedDetail(PurcInquiry inquiry) {
        List<Long> itemIds = new ArrayList<>();
        for (PurcInquiryItem item : inquiry.getInquiryItems()) {
            itemIds.add(item.getId());
        }
        Long[] itemIdArray = new Long[inquiry.getInquiryItems().size()];
        Integer quotedAmount = publicInquiryItemDao.getQuotedAmountBySourceId(itemIds.toArray(itemIdArray));
        Integer replaceQuotedAmount =publicInquiryItemDao.getReplaceQuotedAmountBySourceId(itemIds.toArray(itemIdArray));
        inquiry.setQuotedAmount(quotedAmount);
        inquiry.setReplaceQuotedAmount(replaceQuotedAmount);
        Double minPriceSum = publicInquiryItemDao.getMinPriceSumBySourceId(itemIds.toArray(itemIdArray));
        Double minDeliverySum = publicInquiryItemDao.getMinDeliverySumBySourceId(itemIds.toArray(itemIdArray));
        if (null != minPriceSum) {
            inquiry.setMinPriceSum(new BigDecimal(minPriceSum).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
        } else {
            inquiry.setMinPriceSum(0d);
        }
        if (null != minDeliverySum) {
            inquiry.setMinDeliverySum(new BigDecimal(minDeliverySum).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
        } else {
            inquiry.setMinDeliverySum(0d);
        }
        return inquiry;
    }

    /**
     * 根据id设置询价单明细有效状态
     *
     * @param id    询价单明细id purcInquiryItem
     * @param valid 1有效  0 无效
     * @return
     */
    @CacheEvict(value="inquiry_public", allEntries = true)
    @Override
    public ModelMap deleteItems(Long id, Integer valid) {
        ModelMap map = new ModelMap();
        if (null == id || null == valid) {
            map.put("success", false);
            map.put("message", "传入参数为空");
            return map;
        }
        PurcInquiryItem item = purcInquiryItemDao.findOne(id);
        if (null != item) {
            item.setValid(valid);
            try {
                purcInquiryItemDao.save(item);
                if (valid == 0) {
                    List<InquiryEnRemind> enReminds = inquiryEnRemindDao.findByItemId(id);
                    List<InquiryRemind> reminds = remindDao.findByItemId(id);
                    if (!CollectionUtils.isEmpty(enReminds)) {
                        inquiryEnRemindDao.delete(enReminds);
                    }
                    if (!CollectionUtils.isEmpty(reminds)) {
                        remindDao.delete(reminds);
                    }
                }
                map.put("success", true);
            } catch (Exception e) {
                map.put("success", false);
                map.put("message", e.getMessage());
            }
        } else {
            map.put("success", false);
            map.put("message", "未找到对应询价单明细记录");
        }

        return map;
    }

    /**
     * 将公共询价服务中心的公共询价转成ERP的对应的字段
     *
     * @param notUploadReply 未下载的供应商报价信息
     * @return
     * @author hejq
     * @date 2018-01-14 11:51
     */
    @Override
    public List<InquiryDetail> convertPublicInquiryReply(List<PublicInquiryItem> notUploadReply) {
        List<InquiryDetail> details = new ArrayList<InquiryDetail>();
        if (!CollectionUtils.isEmpty(notUploadReply)) {
            for (PublicInquiryItem item : notUploadReply) {
                details.add(new InquiryDetail(item));
            }
        }
        return details;
    }

    /**
     * 下载完成后更新公共询价服务中心询价单明细的下载状态
     *
     * @param idArray 询价单明细id
     * @author hejq
     * @date 2018-01-14 12:12
     */
    @Override
    public void onReplyUploadSuccess(String[] idArray) {
        List<PublicInquiryItem> items = new ArrayList<>();
        for (String id : idArray) {
            PublicInquiryItem item = publicInquiryItemDao.findOne(Long.parseLong(id));
            if (item != null) {
                item.setBackStatus((short) Status.DOWNLOADED.value());
                item.setErpstatus(Constant.YES);
                items.add(item);
            }
        }
        if (!CollectionUtils.isEmpty(items)) {
            publicInquiryItemDao.save(items);
        }
    }

    /**
     * 下载完成后更新公共询价服务中心询价单明细的下载状态
     *
     * @param idArray 询价单明细id
     * @author hejq
     * @date 2018-01-14 12:12
     */
    @Override
    public void onReplyDownLoadSuccess(String[] idArray) {
        List<PublicInquiryItem> items = new ArrayList<>();
        for (String id : idArray) {
            PublicInquiryItem item = publicInquiryItemDao.findOne(Long.parseLong(id));
            if (item != null) {
                item.setReplySendStatus((short) Status.DOWNLOADED.value());
                items.add(item);
            }
        }
        if (!CollectionUtils.isEmpty(items)) {
            publicInquiryItemDao.save(items);
        }
    }

    /**
     * ERP提交公共询价单之后，更新平台状态，让公共询价服务中心询价单不能进行报价操作
     *
     * @param details 询价信息
     * @author hejq
     * @date 2018-01-14 12:20
     */
    @Override
    public void updateInquiryStatus(List<InquiryDetail> details, Long enuu) {
        if (!CollectionUtils.isEmpty(details)) {
            for (InquiryDetail detail : details) {
                // 更新为过期状态，不能进行报价处理
                String sql = "update purc$puInquiryItems set id_overdue = 1 where id_id = " +
                        "(select id_sourceid from public$inquiryItems where id_id = " + detail.getB2b_id_id() + ")";
                jdbcTemplate.update(sql);
//                // 更新已转报价的询价单明细状态
//                jdbcTemplate.update("update public$inquiryItems set id_checked = 1 where id_id = " + detail.getB2b_id_id());
            }
        }
    }

    /**
     * ERP对报价信息的采纳状态上传到公共询价服务中心
     *
     * @param decides 采纳信息
     */
    @Override
    public void onReplyDecide(List<InquiryDecide> decides, Long enuu) {
        List<PublicInquiryItem> publicInquiryItems = convertInquiryDecide(decides);
        if (!CollectionUtils.isEmpty(publicInquiryItems)) {
            for (PublicInquiryItem inquiryItem : publicInquiryItems) {
                publicInquiryItemDao.updateDecideStatusByID(inquiryItem.getAgreed(), inquiryItem.getDecideStatus(), inquiryItem.getId());
                notifyDecide(inquiryItem, PRODUCERAPP);
            }
        }
    }

    /**
     * 针对供应商，客户对报价结果发送消息通知
     *
     * @param item      询价明细
     * @param sourceApp 应用来源
     */
    @Override
    public void notifyDecide(PublicInquiryItem item, String sourceApp) {
        final String SMS_TYPE = "IM"; //只推送消息
        List<MessageModel> models = new ArrayList<MessageModel>();
        User u = userDao.findOne(null != item.getInquiry().getRecorderUU() ? item.getInquiry().getRecorderUU() : 0L);
        Enterprise e = enterpriseDao.findOne(item.getInquiry().getEnUU());
        Product product = new Product();
        if (null != item.getProductId()) {
            product = productDao.findOne(item.getProductId());
        }
        String cmpCode = product.getCmpCode() != null ? product.getCmpCode() : item.getCmpCode();
        String content = item.getAgreed() == 1 ? "【已采纳】" : "【未采纳】";
        content = content + "您关于“" + cmpCode + "”的报价";
        if (null != e) {
            content = e.getEnName() + content;
        } else if (null != u) {
            content = u.getUserName() + content;
        } else if (null == e && null == u) {
            content = "您关于“" + cmpCode + "”的报价已被";
            String result = item.getAgreed() == 1 ? "【已采纳】" : "【拒绝】";
            content = content + result;
        }
        MessageModel model = new MessageModel();
        model.setType(INQUIRY_DECIDE_TYPE);
        model.setProducerApp(sourceApp);
        model.setConsumerType(CUST_TYPE);
        model.setConsumerApp(CONSUMERAPP);
        model.setContent(content);
        model.setRemark(String.valueOf(item.getInquiry().getId()));
        model.setSourceId(item.getId());
        model.setReceiverEnuu(item.getVendUU());
        model.setReceiverUu(item.getVendUserUU());
        model.setSenderEnuu(item.getInquiry().getEnUU());
        if (null != item.getUserUU() || null != item.getInquiry().getRecorderUU()) {
            model.setSenderUu(item.getUserUU() != null ? item.getUserUU() : item.getInquiry().getRecorderUU());
        } else {
            Enterprise enterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
            model.setSenderUu(null != enterprise ? enterprise.getAdminUU() : 0L);
        }
        model.setSmsType(SMS_TYPE);
        models.add(model);
        if (!CollectionUtils.isEmpty(models)) {
            sendMessage(models);
        }
    }

    /**
     * 将ERP的报价采纳信息转为公共询价服务中心需要的信息
     *
     * @param decides erp报价采纳信息
     * @author hejq
     * @date 2018-01-14 12:25
     */
    @Override
    public List<PublicInquiryItem> convertInquiryDecide(List<InquiryDecide> decides) {
        List<PublicInquiryItem> inquiryItems = new ArrayList<PublicInquiryItem>();
        if (!CollectionUtils.isEmpty(decides)) {
            for (InquiryDecide decide : decides) {
                PublicInquiryItem item = publicInquiryItemDao.findOne(decide.getB2b_id());
                if (null != item) {
                    item.setAgreed(decide.getId_agreed());
                    item.setDecideStatus((short) Status.NOT_UPLOAD.value());
                    inquiryItems.add(item);
                }
            }
        }
        return inquiryItems;
    }

    /**
     * 买家ERP作废询价单后，修改公共询价服务中心里面的状态
     *
     * @param inquiries 询价信息
     * @param enuu      企业UU
     * @author hejq
     * @date 2018-01-14 12:28
     */
    @Override
    public void onReplyInvalid(List<Inquiry> inquiries, final Long enuu) {
        final List<PublicInquiryItem> newInquiryItems = new ArrayList<>();
        for (final Inquiry inquiry : inquiries) {
            ThreadUtils.task(new Runnable() {
                @Override
                public void run() {
                    List<PublicInquiry> purchaseInquiries = publicInquiryDao.findByEnUUAndCode(enuu, inquiry.getIn_code());
                    if (!CollectionUtils.isEmpty(purchaseInquiries)) {
                        PublicInquiry publicInquiry = purchaseInquiries.get(0);
                        for (PublicInquiryItem item : publicInquiry.getInquiryItems()) {
                            item.setStatus((short) Status.DISABLED.value());
                            // 获取作废信息时，更新作废上传状态为 待上传
                            item.setInvalidStatus((short) Status.NOT_UPLOAD.value());
                            synchronized (newInquiryItems) {
                                newInquiryItems.add(item);
                            }
                        }
                    }
                }
            });
        }
        publicInquiryItemDao.save(newInquiryItems);
    }

    /**
     * 将公共询价列表转成需要报价的询价单，类型是公共询价
     *
     * @param id     询价明细id
     * @param enuu   报价企业UU
     * @param useruu 报价人uu
     * @return
     * @author hejq
     * @date 2018-01-14 14:46
     */
    @Override
    public ModelMap transtoInquiry(Long id, Long enuu, Long useruu) {
        ModelMap map = new ModelMap();
        PurcInquiryItem purcInquiryItem = purcInquiryItemDao.findOne(id);
        PurcInquiry purcInquiry = purcInquiryItem.getInquiry();
        // 先判断客户询价单是否存在这张单据
        PublicInquiry inquiry = publicInquiryDao.findBySourceIdAndSourceApp(purcInquiry.getId(), SOURCERAPP);
        if (null == inquiry) {// 新增
            PublicInquiry saleInquiry = new PublicInquiry();
            saleInquiry.setDate(new Date());
            saleInquiry.setCode(purcInquiry.getCode());
            saleInquiry.setEnUU(purcInquiry.getEnUU());
            saleInquiry.setRecorder(purcInquiry.getRecorder());
            saleInquiry.setRecorderUU(purcInquiry.getRecorderUU());
            saleInquiry.setSourceApp(SOURCERAPP);
            saleInquiry.setSourceId(purcInquiry.getId());
            saleInquiry.setInvoice(purcInquiry.getInvoice());
            saleInquiry.setShip(purcInquiry.getShip());
            saleInquiry.setEndDate(purcInquiry.getEndDate());
            if (!CollectionUtils.isEmpty(purcInquiry.getAttachs())) {
                Set<com.uas.ps.inquiry.model.Attach> attachs = new HashSet<com.uas.ps.inquiry.model.Attach>();
                for (com.uas.ps.inquiry.model.Attach attach : purcInquiry.getAttachs()) {
                    com.uas.ps.inquiry.model.Attach newAttach = new com.uas.ps.inquiry.model.Attach();
                    newAttach.setDate(new Date());
                    newAttach.setDescription(attach.getDescription());
                    newAttach.setName(attach.getName());
                    newAttach.setPath(attach.getPath());
                    newAttach.setRelatedKey(attach.getRelatedKey());
                    newAttach.setSize(attach.getSize());
                    newAttach.setRelatedKey(attach.getRelatedKey());
                    attachs.add(newAttach);
                }
                saleInquiry.setAttachs(attachs);

            }
            saleInquiry.setEnvironment(purcInquiry.getEnvironment());
            saleInquiry.setRemark(purcInquiry.getRemark());
            saleInquiry.setInquirytype(purcInquiry.getInquirytype());
            Set<PublicInquiryItem> items = new HashSet<PublicInquiryItem>();
            Short i = 0;
            if (null != purcInquiryItem) {
                PublicInquiryItem item = new PublicInquiryItem();
                Product product = productDao.findOne(purcInquiryItem.getProductId());
                item.setProductId(product.getId());
                item.setInquiry(saleInquiry);
                item.setNumber(i);
                item.setCurrency(purcInquiryItem.getCurrency());
                item.setFromDate(new Date());
                item.setTaxrate(purcInquiryItem.getTaxrate());
                item.setProductId(purcInquiryItem.getProductId());
                item.setStatus((short) Status.NOT_REPLY.value());
                item.setVendUU(enuu);
                item.setVendUserUU(useruu);
                item.setSource(purcInquiryItem.getSource());
                item.setSendStatus((short) Status.NOT_UPLOAD.value());
                item.setSourceId(id);
                List<PublicInquiryReply> replies = new ArrayList<PublicInquiryReply>();
                PublicInquiryReply reply = new PublicInquiryReply();
                reply.setLapQty((double) 0);
                replies.add(reply);
                item.setReplies(replies);
                item.setNeedquantity(purcInquiryItem.getNeedquantity());
                item.setProdCode(purcInquiryItem.getProdCode());
                items.add(item);
                i++;
            }
            List<PublicInquiryItem> purcitems = publicInquiryItemDao.save(items);
            if (purcitems.get(0).getId() != null) {
                map.put("success", "转客户询价单成功");
                map.put("id", purcitems.get(0).getInquiry().getId());
            } else {
                map.put("success", "转客户询价单失败");
            }
        } else {// 插入
            PublicInquiryItem purcItem = publicInquiryItemDao.findByVendUUAndSourceId(enuu, id);
            if (purcItem != null) {
                map.put("info", "询价单已存在");
                map.put("inid", purcItem.getId());
            } else {
                if (!CollectionUtils.isEmpty(purcInquiry.getInquiryItems())) {
                    if (null != purcInquiryItem) {
                        // 涉及到级联存储及ERP转入单据的操作，用主表id存储存在问题，这里先用sql语句直接插入
                        PublicInquiryItem item = new PublicInquiryItem();
                        Product product = productDao.findOne(purcInquiryItem.getProductId());
                        item.setProductId(product.getId());
                        item.setNumber((short) (purcInquiry.getInquiryItems().size() + 1));
                        item.setCurrency(purcInquiryItem.getCurrency());
                        item.setFromDate(new Date());
                        item.setTaxrate(purcInquiryItem.getTaxrate());
                        item.setProductId(purcInquiryItem.getProductId());
                        item.setVendUU(enuu);
                        item.setVendUserUU(useruu);
                        item.setSource(purcInquiryItem.getSource());
                        item.setStatus((short) Status.NOT_REPLY.value());
                        item.setSendStatus((short) Status.NOT_UPLOAD.value());
                        item.setSourceId(id);
                        item.setProdCode(purcInquiryItem.getProdCode());
                        item.setNeedquantity(purcInquiryItem.getNeedquantity());
                        List<PublicInquiryReply> replies = new ArrayList<PublicInquiryReply>();
                        List<PublicInquiryReply> purcReplies = new ArrayList<PublicInquiryReply>();
                        PublicInquiryReply reply = new PublicInquiryReply();
                        reply.setLapQty((double) 0);
                        purcReplies.add(reply);
                        purcReplies = publicInquiryReplyDao.save(purcReplies);
                        replies.addAll(purcReplies);
                        item.setReplies(replies);
                        Long idid = jdbcTemplate.queryForObject("select public$inquiryitems_seq.nextval from dual", Long.class);
                        try {
                            jdbcTemplate.update(
                                    "insert into public$inquiryitems(id_id,id_number,id_currency,id_fromdate,id_taxrate,id_prid,id_venduu,"
                                            + "id_venduseruu,id_sourceapp,id_status,id_sendstatus,id_sourceid,id_inid) values (?,?,?,?,?,?,?,?,?,?,?,?,?)",
                                    idid, item.getNumber(), item.getCurrency(), item.getFromDate(),
                                    item.getTaxrate(), item.getProductId(), item.getVendUU(), item.getVendUserUU(),
                                    item.getSource(), item.getStatus(), item.getSendStatus(),
                                    item.getSourceId(), inquiry.getId());
                        } catch (DataAccessException e) {
                            map.put("info", "转客户询价单失败");
                        }
                        item = publicInquiryItemDao.findOne(idid);
                        //回复清单更新
                        if (replies.size() > 0) {
                            for (PublicInquiryReply reply1 : replies) {
                                jdbcTemplate.update("update public$inquiryreply set ir_idid = " + idid + "where ir_id = " + reply1.getId());
                            }
                        }
                        if (item != null) {
                            map.put("success", "转客户询价单成功");
                            map.put("id", item.getInquiry().getId());
                        } else {
                            map.put("info", "转客户询价单失败");
                        }
                    }
                }
            }

        }
        return map;
    }

    /**
     * 通过分页参数和过滤条件查询报价信息
     *
     * @param info   分页参数
     * @param filter 过滤条件
     * @return
     * @author hejq
     * @date 2018-01-17 10:30
     */
    @Override
    public IPage<PublicInquiryItemInfo> findByPageInfo(final PageInfo info, final SearchFilter filter) {
        Sort sort = new Sort(Sort.Direction.DESC, "date");
        if (info.getOffset() == 0) {
            info.setOffset(info.getPageSize() * (info.getPageNumber() - 1));
        }
        info.setSort(sort);
        if (null != filter) {
            if (filter.getFromDate() != null) {
                info.expression(PredicateUtils.gte("offerTime", new Date(filter.getFromDate()), false));
            }
            if (filter.getEndDate() != null) {
                info.expression(PredicateUtils.lte("offerTime", new Date(filter.getEndDate()), false));
            }
            // 此处的 fromDelivery 作为发布时间过滤
            if (filter.getFromDelivery() != null) {
                info.expression(PredicateUtils.gte("date", new Date(filter.getFromDelivery()), false));
            }
            if (filter.getEndDelivery() != null) {
                info.expression(PredicateUtils.lte("date", new Date(filter.getEndDelivery()), false));
            }
            if (filter.getUserUU() != null) {
                info.expression(PredicateUtils.eq("vendUserUU", filter.getUserUU(), false));
            }
            if (StringUtils.hasText(filter.getKeyword())) {
                SimpleExpression cmpCode = new SimpleExpression("cmpCode", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression brand = new SimpleExpression("inbrand", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression code = new SimpleExpression("inquiry.code", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression title = new SimpleExpression("prodTitle", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression spec = new SimpleExpression("spec", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression inquiryEnName = new SimpleExpression("inquiry.enName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression[] simpleExpressions = new SimpleExpression[]{cmpCode, brand, code, title, inquiryEnName, spec};
                LogicalExpression logical = new LogicalExpression(simpleExpressions, CriterionExpression.Operator.OR);
                info.expression(logical);
            }
        }
        Page<PublicInquiryItemInfo> itemInfoPage = itemInfoDao.findAll(new Specification<PublicInquiryItemInfo>() {
            @Override
            public Predicate toPredicate(Root<PublicInquiryItemInfo> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                query.where(info.getPredicates(root, query, builder));
                return null;
            }
        }, info);
        if (!CollectionUtils.isEmpty(itemInfoPage.getContent())) {
            for (PublicInquiryItemInfo itemInfo : itemInfoPage) {
                String sql = "select at_path,at_name from attachs a left join public$inquiryitems$attach p on a.at_id = p.at_id " +
                        "left join public$inquiryitems i on p.id_id = i.id_id where i.id_id = " + itemInfo.getId();
                List<Map<String, Object>> maps = jdbcTemplate.queryForList(sql);
                if (!CollectionUtils.isEmpty(maps)) {
                    Map<String, Object> map = maps.get(0);
                    itemInfo.setAttachUrl(map.get("at_path").toString());
                    itemInfo.setAttachName(map.get("at_name").toString());
                }
                if (null == itemInfo.getProductId()) {
                    Product product = new Product();
                    product.setTitle(itemInfo.getProdTitle());
                    product.setCmpCode(itemInfo.getCmpCode());
                    product.setBrand(itemInfo.getInbrand());
                    product.setSpec(itemInfo.getSpec());
                    product.setCode(itemInfo.getProdCode());
                    if (null == product.getCode()) {
                        product.setCode(itemInfo.getCmpCode());
                    }
                    itemInfo.setProduct(product);
                }
            }
        }
        return IPageUtils.covert(itemInfoPage);
    }

    @Override
    public Page<PublicInquiryItemInfo> findListByPageInfo(final PageInfo info, SearchFilter filter) {
        Sort sort = new Sort(Sort.Direction.DESC, "date");
        if (info.getOffset() == 0) {
            info.setOffset(info.getPageSize() * (info.getPageNumber() - 1));
        }
        info.setSort(sort);
        return itemInfoDao.findAll(new Specification<PublicInquiryItemInfo>() {
            @Override
            public Predicate toPredicate(Root<PublicInquiryItemInfo> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                query.where(info.getPredicates(root, query, builder));
                return null;
            }
        }, info);
    }

    /**
     * 查询公共询价列表信息
     *
     * @param info   分页参数
     * @param filter 过滤条件
     *
     * @return
     */
    @Override
    public IPage<PurcInquiryItemInfo> findTodoByPageInfo(final PageInfo info, final SearchFilter filter) {
        long start = System.currentTimeMillis();
        if (null != filter && !"{}".equals(JSON.toJSONString(filter))) {
            Long enUU = filter.getEnUU();

            // 过滤条件处理
           setFilter(info, filter);

            Page<PurcInquiryItemInfo> orders = inquiryItemInfoDao.findAll(new Specification<PurcInquiryItemInfo>() {
                @Override
                public Predicate toPredicate(Root<PurcInquiryItemInfo> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                    query.where(info.getPredicates(root, query, builder));
                    return null;
                }
            }, info);
            log.info("filter-查询询价单列表，耗时:{}", (System.currentTimeMillis() - start));

            // 设置报价信息并返回
            return covert(orders, enUU);
        } else {
            // 查询询价单语句
            String sql = "SELECT * FROM ( SELECT * FROM purc$puinquiryitems WHERE id_valid = 1 ORDER BY id_date DESC LIMIT ?,? ) a " +
                    "LEFT JOIN `purc$puinquiry` b ON a.id_inid = b.in_id";
            List<PurcInquiryItemInfo> orders = jdbcTemplate.query(sql, getRowMapper(),
                    info.getPageSize() * (info.getPageNumber() - 1), info.getPageSize() * info.getPageNumber());
            log.info("查询询价单耗时：{}", (System.currentTimeMillis() - start));
            start = System.currentTimeMillis();
            String countSql = "select count(1) from purc$puinquiryitems where id_valid = 1";
            Integer count = jdbcTemplate.queryForObject(countSql, Integer.class);
            log.info("查询条数耗时：{}", (System.currentTimeMillis() - start));

            IPage<PurcInquiryItemInfo> iPage = new IPage<>();
            iPage.setTotalElements(count);
            iPage.setSize(info.getPageSize());
            iPage.setNumber(info.getPageNumber());
            iPage.setContent(orders);
            return iPage;
        }

    }

    /**
     * 过滤条件处理
     * @param info  分页参数
     * @param filter  过滤条件
     */
    private void setFilter(PageInfo info, SearchFilter filter) {
        Sort sort = new Sort(Sort.Direction.DESC, "id");
        info.setSort(sort);
        if (filter.getFromDate() != null) {
            info.expression(PredicateUtils.gte("date", new Date(filter.getFromDate()), false));
        }
        if (filter.getEndDate() != null) {
            info.expression(PredicateUtils.lte("date", new Date(filter.getEndDate()), false));
        }
        if (!CollectionUtils.isEmpty(filter.getDistribute())) {
            info.expression(PredicateUtils.in("enuu", filter.getDistribute(), false));
        }
        if (!StringUtils.isEmpty(filter.getIsAll()) && filter.getIsAll() == 0 && !StringUtils.isEmpty(filter.getEnUU())) {
            info.expression(PredicateUtils.ne("enuu", filter.getEnUU(), false));
        }
        if (!StringUtils.isEmpty(filter.getKeyword())) {
            SimpleExpression cmpCode = new SimpleExpression("cmpCode", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
            SimpleExpression brand = new SimpleExpression("inbrand", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
            SimpleExpression title = new SimpleExpression("prodTitle", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
            SimpleExpression spec = new SimpleExpression("spec", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
            SimpleExpression enName = new SimpleExpression("inquiry.enName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
            SimpleExpression[] simpleExpressions = null;
            if (!StringUtils.isEmpty(filter.getIsLogin()) && filter.getIsLogin() == 1) {
                simpleExpressions = new SimpleExpression[]{cmpCode, brand, title, enName, spec};
            } else {
                simpleExpressions = new SimpleExpression[]{cmpCode, brand, title, spec};
            }
            LogicalExpression logical = new LogicalExpression(simpleExpressions, CriterionExpression.Operator.OR);
            info.expression(logical);
        }
    }

    /**
     * 实体转换
     * @return
     */
    private RowMapper<PurcInquiryItemInfo> getRowMapper() {
        return new RowMapper<PurcInquiryItemInfo>() {

            @Override
            public PurcInquiryItemInfo mapRow(ResultSet rs, int rowNum) throws SQLException {
                PurcInquiryItemInfo item = new PurcInquiryItemInfo();
                item.setId(rs.getLong("id_id"));
                item.setNumber(rs.getShort("id_number"));
                item.setUserUU(rs.getLong("id_useruu"));
                item.setUserName(rs.getString("id_username"));
                item.setUserTel(rs.getString("id_usertel"));
                item.setProductId(rs.getLong("id_prid"));
                item.setProdCode(rs.getString("id_prodcode"));
                item.setProdTitle(rs.getString("id_prodtitle"));
                item.setSpec(rs.getString("id_prodspec"));
                item.setInbrand(rs.getString("id_inbrand"));
                item.setCurrency(rs.getString("id_currency"));
                item.setTaxrate(rs.getFloat("id_taxrate"));
                item.setFromDate(rs.getDate("id_date"));
                item.setStatus(rs.getShort("id_status"));
                item.setOverdue(rs.getShort("id_overdue"));
                item.setSource(rs.getString("id_source"));
                item.setNeedquantity(rs.getDouble("id_needquantity"));
                item.setSourceid(rs.getLong("id_sourceid"));
                item.setDate(rs.getDate("id_date"));
                item.setEnuu(rs.getLong("id_enuu"));
                item.setEndDate(rs.getDate("id_enddate"));
                item.setOfferAmount(rs.getInt("id_offeramount"));
                item.setAgreed(rs.getShort("id_agreed"));
                item.setValid(rs.getInt("id_valid"));

                PurcInquiryInfo purcInquiryInfo = new PurcInquiryInfo();
                purcInquiryInfo.setId(rs.getLong("in_id"));
                purcInquiryInfo.setEnUU(rs.getLong("in_enuu"));
                purcInquiryInfo.setEnName(rs.getString("in_enname"));
                purcInquiryInfo.setRecorderUU(rs.getLong("in_recorderuu"));
                purcInquiryInfo.setCode(rs.getString("in_code"));
                purcInquiryInfo.setDate(rs.getDate("in_date"));
                purcInquiryInfo.setRecorder(rs.getString("in_recorder"));
                purcInquiryInfo.setEndDate(rs.getDate("in_enddate"));
                purcInquiryInfo.setRemark(rs.getString("in_remark"));
                purcInquiryInfo.setEnvironment(rs.getString("in_environment"));
                purcInquiryInfo.setEnteryStatus(rs.getInt("in_enterystatus"));
                purcInquiryInfo.setShip(rs.getString("in_ship"));
                purcInquiryInfo.setInvoice(rs.getShort("in_invoice"));
                purcInquiryInfo.setIfTax(rs.getShort("in_iftax"));
                purcInquiryInfo.setCurrency(rs.getString("in_currency"));
                purcInquiryInfo.setInquirytype(rs.getString("in_inquirytype"));
                purcInquiryInfo.setSourceapp(rs.getString("in_sourceapp"));
                purcInquiryInfo.setErpid(rs.getLong("in_erpid"));
                purcInquiryInfo.setErpdate(rs.getDate("in_erpdate"));
                purcInquiryInfo.setAmount(rs.getInt("in_amount"));
                purcInquiryInfo.setSpec(rs.getString("in_spec"));
                purcInquiryInfo.setCount(rs.getInt("in_count"));

                item.setInquiry(purcInquiryInfo);
                return item;
            }

        };
    }

//    /**
//     * 将数据库查询的数据进行封装处理
//     *
//     * @param orders 查询的数据信息
//     * @return
//     */
//    private IPage<PurcInquiryItemInfo> covert(Page<PurcInquiryItemInfo> orders, Long enuu) {
//        long start = System.currentTimeMillis();
//        if (!CollectionUtils.isEmpty(orders.getContent()) && null != enuu) {
//            for (PurcInquiryItemInfo itemInfo : orders.getContent()) {
//                PublicInquiryItem item = publicInquiryItemDao.findByVendUUAndSourceId(enuu, itemInfo.getId());
//                itemInfo.setQuoted(Constant.NO);
//                if (null != item) {
//                    itemInfo.setAgreed(item.getAgreed());
//                    itemInfo.setQuteId(item.getId());
//                    itemInfo.setQuoted(Constant.YES);
//                }
//                Product product = new Product();
//                product.setTitle(itemInfo.getProdTitle());
//                product.setCmpCode(itemInfo.getCmpCode());
//                product.setBrand(itemInfo.getInbrand());
//                product.setSpec(itemInfo.getSpec());
//                product.setCode(itemInfo.getProdCode());
//                if (null == product.getCode()) {
//                    product.setCode(itemInfo.getCmpCode());
//                }
//                itemInfo.setProduct(product);
//            }
//        }
//        log.info("转换数据耗时：{}", (System.currentTimeMillis() - start));
//        return IPageUtils.covert(orders);
//    }


    /**
     * 将数据库查询的数据进行封装处理
     *
     * @param orders 查询的数据信息
     * @return
     */
    private IPage<PurcInquiryItemInfo> covert(Page<PurcInquiryItemInfo> orders, Long enuu) {
        long start = System.currentTimeMillis();
        List<Long> sourceIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(orders.getContent()) && null != enuu) {
            for (PurcInquiryItemInfo itemInfo : orders.getContent()) {
                sourceIds.add(itemInfo.getId());
                Product product = new Product();
                product.setTitle(itemInfo.getProdTitle());
                product.setCmpCode(itemInfo.getCmpCode());
                product.setBrand(itemInfo.getInbrand());
                product.setSpec(itemInfo.getSpec());
                product.setCode(itemInfo.getProdCode());
                if (null == product.getCode()) {
                    product.setCode(itemInfo.getCmpCode());
                }
                itemInfo.setProduct(product);
            }

            // 查询当前企业的报价单
            List<PublicInquiryItem> items = publicInquiryItemDao.findByVendUUAndSourceIdIn(enuu, sourceIds);

            //map 报价单信息
            HashMap<Long, PublicInquiryItem> map = new HashMap<>(50);
            for (PublicInquiryItem item : items) {
                // 报价单sourceId(询价单明细id)作为key
                map.put(item.getSourceId(), item);
            }

            for (PurcInquiryItemInfo itemInfo : orders.getContent()) {
                itemInfo.setQuoted(Constant.NO);
                if (null != map.get(itemInfo.getId())) {
                    itemInfo.setAgreed(map.get(itemInfo.getId()).getAgreed());
                    itemInfo.setQuteId(map.get(itemInfo.getId()).getId());
                    itemInfo.setQuoted(Constant.YES);
                }
            }
        }
        log.info("filter-转换数据耗时：{}", (System.currentTimeMillis() - start));
        return IPageUtils.covert(orders);
    }

    /**
     * 根据传入条件封装查询条件
     *
     * @param pageInfo 分页参数
     * @param state    过滤条件
     * @return
     */
    @Override
    public PageInfo covert(PageInfo pageInfo, String state, Short overdue) {
        Sort sort = new Sort(Sort.Direction.DESC, "date");
        if (pageInfo.getOffset() == 0) {
            pageInfo.setOffset(pageInfo.getPageSize() * (pageInfo.getPageNumber() - 1));
        }
        pageInfo.setSort(sort);
        if (state.equals(OrderStatus.todo.name())) {
            pageInfo.filter("status", Status.NOT_REPLY.value());
            pageInfo.filter("overdue", Constant.NO);
        } else if (state.equals(OrderStatus.done.name())) {
            if (StringUtils.isEmpty(overdue) || !overdue.equals(Constant.YES)) {
                pageInfo.filter("overdue", Constant.NO);
            }
            pageInfo.filter("status", Status.REPLIED.value());
            pageInfo.filter("decideStatus", Status.UNAUDIT.value());
        } else if (state.equals(OrderStatus.end.name())) {
            pageInfo.filter("overdue", Constant.YES);
        } else if (state.equals(OrderStatus.invalid.name())) {
            SimpleExpression overdue1 = new SimpleExpression("overdue", Constant.NO, CriterionExpression.Operator.NE, true);
            SimpleExpression invalid = new SimpleExpression("invalid", Constant.NO, CriterionExpression.Operator.NE, true);
            SimpleExpression[] simpleExpressions = new SimpleExpression[]{invalid, overdue1};
            LogicalExpression logical = new LogicalExpression(simpleExpressions, CriterionExpression.Operator.OR);
            pageInfo.expression(logical);
        } else if (state.equals(OrderStatus.agreed.name())) {
            pageInfo.filter("agreed", Constant.YES);
            pageInfo.filter("status", Status.REPLIED.value());
        } else if (state.equals(OrderStatus.refused.name())) {
            pageInfo.filter("agreed", Constant.NO);
            pageInfo.filter("status", Status.REPLIED.value());
        }
        return pageInfo;
    }

    /**
     * 获取公共询价列表信息
     *
     * @param enuu 当前企业UU
     * @param size 大小
     * @return
     */
    @Override
    public List<InquiryMessage> getMessageList(Long enuu, Integer size) {
        if (size == 0 || size == null) {
            size = 10;
        }
        String sql = "select * from (select p.id_id id, p.id_prodtitle prodTitle, p.id_date as date, i.in_enuu enuu from purc$puinquiryitems p left join purc$puinquiry i on i.in_id = p.id_inid where 1=1";
        if (enuu != null) {
            sql = sql + " and i.in_enuu <> " + enuu;
        }
        sql = sql + " and COALESCE(p.id_overdue, 0) <> 1 order by p.id_id desc) i limit " + size;
        List<PurcInquiryItem> items = jdbcTemplate.query(sql, new BeanPropertyRowMapper<PurcInquiryItem>(PurcInquiryItem.class));
        List<InquiryMessage> messages = new ArrayList<InquiryMessage>();
        if (!CollectionUtils.isEmpty(items)) {
            for (PurcInquiryItem item : items) {
                InquiryMessage message = new InquiryMessage();
                if (null != item.getEnuu()) {
                    Enterprise enterprise = enterpriseDao.findOne(item.getEnuu());
                    message.setEnName(enterprise.getEnName());
                }
                item = purcInquiryItemDao.findOne(item.getId());
                message.setPordName(item.getProdTitle());
                message.setInid(item.getId());
                PublicInquiryItem publicInquiryItem = publicInquiryItemDao.findByVendUUAndSourceId(enuu, item.getId());
                if (null != publicInquiryItem) {
                    message.setQuteId(publicInquiryItem.getId());
                }
                message.setLastest(isToday(item.getDate()));
                messages.add(message);
            }
        }
        return messages;
    }

    /**
     * 判断该条单据是否当天单据
     *
     * @param date 询价时间
     * @return
     */
    private boolean isToday(Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String inDate = sdf.format(date);
        String nowDate = sdf.format(new Date());
        return inDate.equals(nowDate);
    }

    /**
     * 商城-采购排行榜
     *
     * @return
     */
    @Override
    public List<Map<String, Object>> findCodeRankingList() {
        String name = "采购排行榜";
        InquiryData inquiryData = inquiryDataDao.findByName(name);
        List<Map<String, Object>> result = FlexJsonUtils.fromJsonArray(inquiryData.getData(), HashMap.class);
        return result;
    }

    /**
     * 通过明细id查询询价详情
     *
     * @param itemId 明细id
     * @return
     */
    @Override
    public PurcInquiryItemInfo findItemByItemId(Long itemId, Long enuu) {
        PurcInquiryItemInfo itemInfo = inquiryItemInfoDao.findOne(itemId);
        PublicInquiryItem item = publicInquiryItemDao.findByVendUUAndSourceId(enuu, itemInfo.getId());
        itemInfo.setQuoted(Constant.NO);
        if (null != item) {
            itemInfo.setAgreed(item.getAgreed());
            itemInfo.setQuteId(item.getId());
            itemInfo.setQuoted(Constant.YES);
        }
        if (null == itemInfo.getProductId()) {
            Product product = new Product();
            product.setTitle(itemInfo.getProdTitle());
            product.setCmpCode(itemInfo.getCmpCode());
            product.setBrand(itemInfo.getInbrand());
            product.setSpec(itemInfo.getSpec());
            itemInfo.setProduct(product);
        }
        return itemInfo;
    }

    /**
     * 通过id查询单个已转报价的信息
     *
     * @param id
     * @return
     */
    @Override
    public PublicInquiryItemInfo findOneInfo(Long id, Long enuu, Long useruu) {
        PublicInquiryItemInfo itemInfo = itemInfoDao.findOne(id);
        if (!itemInfo.getVendUU().equals(enuu)) {
            throw new IllegalOperatorException("访问受限");
        }
        return itemInfo;
    }

    /**
     * 将推荐信息转成UU互联需要的数据
     *
     * @param content 推荐信息
     * @return
     */
    @Override
    public List<PurcInquiryItemInfo> covertRemindToInquiry(List<InquiryRemind> content) {
        List<PurcInquiryItemInfo> infoList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(content)) {
            for (InquiryRemind remind : content) {
                PurcInquiryItemInfo itemInfo = inquiryItemInfoDao.findOne(remind.getItemId());
                if (null != itemInfo) {
                    Product product = new Product();
                    product.setTitle(itemInfo.getProdTitle());
                    product.setCmpCode(itemInfo.getCmpCode());
                    product.setCode(itemInfo.getCmpCode());
                    product.setBrand(itemInfo.getInbrand());
                    product.setSpec(itemInfo.getSpec());
                    itemInfo.setProduct(product);
                    if (remind.getStatus().equals(Status.NOTALLOW.value())) {
                        itemInfo.setAgreed(Constant.NO);
                    } else if (remind.getStatus().equals(Status.ALLOW.value())) {
                        itemInfo.setAgreed(Constant.YES);
                    }
                    itemInfo.setQuoted(Constant.NO);
                    if (remind.getStatus().equals(Status.SUBMITTED.value())) {
                        itemInfo.setQuoted(Constant.YES);
                        itemInfo.setQuteId(remind.getNewId());
                    }
                    if (null != itemInfo.getRemainingTime() && itemInfo.getRemainingTime() <= 0) {
                        itemInfo.setOverdue(Constant.YES);
                    }
                    infoList.add(itemInfo);
                }
            }
        }
        return infoList;
    }

    /**
     * 针对供应商，客户将拒绝理由发送信息给供应商
     *
     * @param item      询价明细
     * @param sourceApp 应用来源
     * @throws Exception
     */
    @Override
    public Page<MessageModel> refuseMessage(PublicInquiryItem item, String sourceApp) {
        if (null != item.getVendUU() && null != item.getVendUserUU()) {
            final String SMS_TYPE = "SM"; //发送信息
            List<MessageModel> models = new ArrayList<MessageModel>();
            String Buyer = item.getInquiry().getEnName() == null ? item.getInquiry().getRecorder() : item.getInquiry().getEnName();
            String cmpCode = item.getCmpCode();
            String cmp = cmpCode.length() > 10 ? cmpCode.substring(0, 10).concat("...") : cmpCode;
            String content = "因" + item.getRefusereason() + "，" + Buyer + "【未采纳】您关于“型号：" + cmp + "”的报价。期待与您的下次合作！";
            MessageModel model = new MessageModel();
            model.setType(INQUIRY_DECIDE_TYPE_MALL);
            model.setProducerApp(sourceApp);
            model.setConsumerType(CUST_TYPE);
            model.setConsumerApp(CONSUMERAPP);
            model.setContent(content);
            model.setRemark(String.valueOf(item.getInquiry().getId()));
            model.setSourceId(item.getId());
            model.setReceiverEnuu(item.getVendUU());
            model.setReceiverUu(item.getVendUserUU());
            model.setSenderEnuu(item.getInquiry().getEnUU());
            if (null != item.getUserUU() || null != item.getInquiry().getRecorderUU()) {
                model.setSenderUu(item.getUserUU() != null ? item.getUserUU() : item.getInquiry().getRecorderUU());
            } else {
                Enterprise enterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
                model.setSenderUu(null != enterprise ? enterprise.getAdminUU() : 0L);
            }
            model.setSmsType(SMS_TYPE);
            models.add(model);
            if (!CollectionUtils.isEmpty(models)) {
                sendMessage(models);
            }
        }
        return null;

    }

    /**
     * 针对供应商，把客户采纳报价的消息发送给供应商
     *
     * @param item      询价明细
     * @param sourceApp 应用来源
     * @throws Exception
     */
    @Override
    public Page<MessageModel> adoptMessage(PublicInquiryItem item, String sourceApp) {
        if (null != item.getVendUU() && null != item.getVendUserUU()) {
            final String SMS_TYPE = "SM"; //发送信息
            List<MessageModel> models = new ArrayList<MessageModel>();
            String cmpCode = item.getCmpCode();
            String cmp = cmpCode.length() > 10 ? cmpCode.substring(0, 10).concat("...") : cmpCode;
            String content = item.getInquiry().getEnName() + "【已采纳】您关于'型号：'" + cmp + "的报价，期待与您的合作！";
            MessageModel model = new MessageModel();
            model.setType(INQUIRY_DECIDE_TYPE_MALL);
            model.setProducerApp(sourceApp);
            model.setConsumerType(CUST_TYPE);
            model.setConsumerApp(CONSUMERAPP);
            model.setContent(content);
            model.setRemark(String.valueOf(item.getInquiry().getId()));
            model.setSourceId(item.getId());
            model.setReceiverEnuu(item.getVendUU());
            model.setReceiverUu(item.getVendUserUU());
            model.setSenderEnuu(item.getInquiry().getEnUU());
            if (null != item.getUserUU() || null != item.getInquiry().getRecorderUU()) {
                model.setSenderUu(item.getUserUU() != null ? item.getUserUU() : item.getInquiry().getRecorderUU());
            } else {
                Enterprise enterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
                model.setSenderUu(null != enterprise ? enterprise.getAdminUU() : 0L);
            }
            model.setSmsType(SMS_TYPE);
            models.add(model);
            if (!CollectionUtils.isEmpty(models)) {
                sendMessage(models);
            }
        }
        return null;

    }

    @Override
    public Page<MessageModel> ReceiveMessage(PublicInquiryItem item, String sourceApp) {
        if (null != item.getVendUU() && null != item.getVendUserUU() && !StringUtils.isEmpty(item.getCmpCode())) {
            final String SMS_TYPE = "SM"; //发送信息
            List<MessageModel> models = new ArrayList<MessageModel>();
            String cmpCode = item.getCmpCode();
            String cmp = cmpCode.length() > 10 ? cmpCode.substring(0, 10).concat("...") : cmpCode;
            String Name= enterpriseDao.findOne(item.getVendUU()).getEnName();
            String enuuName =item.getVendName() != null ? item.getVendName(): Name;
            String content = "您收到了" + enuuName + "关于“型号：" + cmp + "“的报价，马上去看一看吧！";
            MessageModel model = new MessageModel();
            model.setType(INQUIRY_TYPE_MALL);
            model.setProducerApp(sourceApp);
            model.setConsumerType(CUST_TYPE);
            model.setConsumerApp(CONSUMERAPP);
            model.setContent(content);
            model.setRemark(String.valueOf(item.getInquiry().getId()));
            model.setSourceId(item.getId());
            model.setReceiverEnuu(item.getInquiry().getEnUU());
            model.setReceiverUu(item.getInquiry().getRecorderUU());
            model.setSenderEnuu(item.getVendUU());
            if (null != item.getUserUU() || null != item.getInquiry().getRecorderUU()) {
                model.setSenderUu(item.getUserUU() != null ? item.getUserUU() : item.getInquiry().getRecorderUU());
            } else {
                Enterprise enterprise = enterpriseDao.findOne(item.getInquiry().getEnUU());
                model.setSenderUu(null != enterprise ? enterprise.getAdminUU() : 0L);
            }
            model.setSmsType(SMS_TYPE);
            models.add(model);
            if (!CollectionUtils.isEmpty(models)) {
                sendMessage(models);
            }
        }
        return null;


    }

    @Override
    public ModelMap getPurcInquiryItemCount() {
        ModelMap result = null;
        try {
            result = new ModelMap();
            Long count = inquiryItemInfoDao.count();
            result.put("count", count);
        } catch (Exception e) {
            e.printStackTrace();

        }
        return result;
    }

    /**
     * 查询某年某月的询价明细总数
     *
     * @param month
     * @return
     */
    @Override
    public ModelMap countByMonth(Integer year, Integer month) {
        ModelMap result = null;
        try {
            result = new ModelMap();
            Long count = inquiryItemInfoDao.countByMonth(year, month);
            result.put("count", count);
            result.put("success", "true");
        } catch (Exception e) {
            result.put("message", e.getMessage());
            result.put("success", false);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 根据年份和月份字符串获取条数
     *
     * @param year   年份
     * @param months 月份字符串 6,7,8
     * @return map
     */
    @Override
    public List<ModelMap> countByMonths(Integer year, String months) {
        List<ModelMap> results = new ArrayList<>();
        try {
            String[] monthArray = months.split(",");
            for (String month : monthArray) {
                ModelMap result = new ModelMap();
                Long count = inquiryItemInfoDao.countByMonth(year, Integer.valueOf(month));
                result.put("count", count);
                result.put("success", "true");
                result.put("month", Integer.valueOf(month));
                results.add(result);
            }
        } catch (Exception e) {
            ModelMap result = new ModelMap();
            result.put("message", e.getMessage());
            result.put("success", false);
            results.add(result);
            e.printStackTrace();
        }
        return results;
    }

    /**
     * 根据enUU获取
     *
     * @param enUU 报价企业UU
     * @return
     */
    @Override
    public List<PublicInquiryItem> findDecideNotUploadReply(Long enUU) {
        String sql = "select id_id id, id_sourceid sourceId, id_agreed agreed, id_refusereason refusereason, id_status status from public$inquiryitems " +
                "where id_venduu = " + enUU + " and id_decidedownstatus = " + Status.NOT_UPLOAD.value() + " limit 200";
        return jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(PublicInquiryItem.class));
    }

    /**
     * 已被采纳或拒绝的报价单明细 上传买方UAS之后，回调
     *
     * @param ids 报价单明细ids
     * @param enUU 报价企业UU
     * @return
     */
    @Override
    public ModelMap downloadDecideBack(List<Long> ids, Long enUU) {
        List<PublicInquiryItem> items = publicInquiryItemDao.findAll(ids);
        List<PublicInquiryItem> saveList = new ArrayList<>();
        ModelMap map = new ModelMap();
        for (PublicInquiryItem item : items) {
            if (enUU.equals(item.getVendUU())) {
                item.setDecideDownStatus((short) Status.DOWNLOADED.value());
                saveList.add(item);
            }
        }
        publicInquiryItemDao.save(saveList);
        map.put("success", true);
        return map;
    }

    @Override
    public ModelMap getInquiryCountData(String startDate, String endDate) {
        ModelMap result = new ModelMap();
        if (StringUtils.isEmpty(startDate) && StringUtils.isEmpty(endDate)) {
            throw new IllegalOperatorException("缺失参数");
        }
        // 查询询价数
        Integer inquiryAmount = purcInquiryItemDao.countInquiryAmount(startDate, endDate);
        result.put("inquiryAmount", inquiryAmount);
        // 查询询价用户数
        Integer inquiryUserAmount = purcInquiryItemDao.countInquiryUserAmount(startDate, endDate);
        result.put("inquiryUserAmount", inquiryUserAmount);
        // 查询报价用户
        Integer offerAmount = publicInquiryItemDao.countOfferAmount(startDate, endDate);
        result.put("offerAmount", offerAmount);
        // 查询报价数
        Integer offerEnAmount = publicInquiryItemDao.countOfferEnAmount(startDate, endDate);
        result.put("offerEnAmount", offerEnAmount);
        result.put("success", true);
        return result;
    }

    /**
     * 查询个人不同型号发布询价数量统计
     *
     * @param enuu      企业uu
     * @param useruu    用户uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Override
    public ModelMap sumPersonalPublish(Long enuu, Long useruu, Long starttime, Long endtime) {
        ModelMap map = new ModelMap();
        if (null == enuu) {
            map.put("count", purcInquiryItemDao.sumPersonalPublishWithoutEnuu(useruu, new Date(starttime), new Date(endtime)));
        } else {
            map.put("count", purcInquiryItemDao.sumPersonalPublish(enuu, useruu, new Date(starttime), new Date(endtime)));
        }
        return map;
    }

    /**
     * 查询企业不同型号发布询价数量统计
     *
     * @param enuu      企业uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Override
    public ModelMap sumEnterprisePublish(Long enuu, Long starttime, Long endtime) {
        ModelMap map = new ModelMap();
        map.put("count", purcInquiryItemDao.sumEnterprisePublish(enuu, new Date(starttime), new Date(endtime)));
        return map;
    }

    /**
     * 查询个人报价询价数量统计
     *
     * @param enuu      企业uu
     * @param useruu    用户uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Override
    public ModelMap sumPersonalReply(Long enuu, Long useruu, Long starttime, Long endtime) {
        ModelMap map = new ModelMap();
        map.put("count", purcInquiryItemDao.sumPersonalReply(enuu, useruu, new Date(starttime), new Date(endtime)));
        return map;
    }

    /**
     * 查询企业回复询价数量统计
     *
     * @param enuu      企业uu
     * @param starttime 开始时间
     * @param endtime   结束时间
     * @return 查询结果
     */
    @Override
    public ModelMap sumEnterpriseReply(Long enuu, Long starttime, Long endtime) {
        ModelMap map = new ModelMap();
        map.put("count", purcInquiryItemDao.sumEnterpriseReply(enuu, new Date(starttime), new Date(endtime)));
        return map;
    }

    /**
     * 获取今天和昨天的询价和报价统计
     *
     * @param time 时间
     * @return 统计结果
     */
    @Override
    public HashMap<String, Object> sumRecentPublishWithTime(Long time) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        // 获取上周用户最大uu号
        ModelMap user = SsoUtils.getLastWeekMaxUU();
        if (null == user) {
            throw new RuntimeException("未获取到上周最后注册用户UU");
        }
        Integer userUU = (Integer) user.get("content");
        if (null == userUU) {
            throw new RuntimeException("获取到上周最后注册用户UU为空");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 今天
        String date = sdf.format(new Date(time));
        // 昨天
        String yesterdayString = sdf.format(new Date(time - 1000 * 60 * 60 * 24));
        // 昨天统计
        ModelMap yesterday = new ModelMap();
        // 今天统计
        ModelMap today = new ModelMap();
        // 前一天老用户统计
        Integer yesterdayOldPublish = purcInquiryItemDao.sumOldInquiryCountByUserUUAndDate(userUU, yesterdayString);
        Integer yesterdayOldReply = publicInquiryItemDao.sumOldReplyCountByUserUUAndDate(userUU, yesterdayString);
        // 前一天全部
        Integer yesterdayTotalPublish = purcInquiryItemDao.sumInquiryCountByDate(yesterdayString);
        Integer yesterdayTotalReply = publicInquiryItemDao.sumReplyCountByDate(yesterdayString);
        // 日期当天老用户统计
        Integer todayOldPublish = purcInquiryItemDao.sumOldInquiryCountByUserUUAndDate(userUU, date);
        Integer todayOldReply = publicInquiryItemDao.sumOldReplyCountByUserUUAndDate(userUU, date);
        // 日期当天全部
        Integer todayPublishTotal = purcInquiryItemDao.sumInquiryCountByDate(date);
        Integer todayTotalReply = publicInquiryItemDao.sumReplyCountByDate(date);

        // 前一天新老用户 发布数量设置
        yesterday.put("yesterdayOldPublish", yesterdayOldPublish);
        yesterday.put("yesterdayNewPublish", yesterdayTotalPublish - yesterdayOldPublish);
        yesterday.put("yesterdayTotalPublish", yesterdayTotalPublish);

        // 前一天新老用户 报价数量设置
        yesterday.put("yesterdayOldReply", yesterdayOldReply);
        yesterday.put("yesterdayNewReply", yesterdayTotalReply - yesterdayOldReply);
        yesterday.put("yesterdayTotalReply", yesterdayTotalReply);

        // 日期当天新老用户 发布和报价数量设置
        today.put("todayOldPublish", todayOldPublish);
        today.put("todayNewPublish", todayPublishTotal - todayOldPublish);
        today.put("todayTotalPublish", todayPublishTotal);

        // 日期当天新老用户 报价数量设置
        today.put("todayOldReply", todayOldReply);
        today.put("todayNewReply", todayTotalReply - todayOldReply);
        today.put("todayTotalReply", todayTotalReply);
        result.put("yesterday", yesterday);
        result.put("today", today);
        return result;
    }

    /**
     * 获取区间时间内的询价和报价统计
     *
     * @param starttime 起始时间
     * @param endtime   起始时间
     * @return 统计结果
     */
    @Override
    public ModelMap sumPublishWithTimeScope(Long starttime, Long endtime) throws Exception {
        ModelMap map = new ModelMap();
        // 获取上周用户最大uu号
        ModelMap user = SsoUtils.getLastWeekMaxUU();
        if (null == user) {
            throw new RuntimeException("未获取到上周最后注册用户UU");
        }
        Integer userUU = (Integer) user.get("content");
        if (null == userUU) {
            throw new RuntimeException("获取到上周最后注册用户UU为空");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 今天
        String startDate = sdf.format(new Date(starttime));
        // 昨天
        String endDate = sdf.format(new Date(endtime));
        // 老用户统计
        Integer oldPublish = purcInquiryItemDao.sumOldInquiryCountWithDateScope(startDate, endDate, userUU);
        Integer oldReply = publicInquiryItemDao.sumOldReplyCountWithDateScope(startDate, endDate, userUU);
        // 全部
        Integer totalPublish = purcInquiryItemDao.sumInquiryCountWithTimeScope(startDate, endDate);
        Integer totalReply = publicInquiryItemDao.sumReplyCountWithTimeScope(startDate, endDate);

        // 新老用户 发布数量设置
        map.put("oldPublish", oldPublish);
        map.put("newPublish", totalPublish - oldPublish);
        map.put("totalPublish", totalPublish);

        // 新老用户 报价数量设置
        map.put("oldReply", oldReply);
        map.put("newReply", totalReply - oldReply);
        map.put("totalReply", totalReply);
        return map;
    }

    /**
     * 根据企业uu查询发布询价条数
     *
     * @param enuu 企业UU
     * @return
     */
    @Override
    public ModelMap sumCountWithEnuu(Long enuu) {
        ModelMap map = new ModelMap();
        map.put("count", purcInquiryItemDao.countByEnuu(enuu));
        return map;
    }
}
