package com.uas.ps.inquiry.controller;

import com.alibaba.fastjson.JSON;
import com.uas.ps.inquiry.BaseJunitTest;
import com.uas.ps.inquiry.model.Attach;
import com.uas.ps.inquiry.model.PublicInquiry;
import com.uas.ps.inquiry.model.PublicInquiryItem;
import com.uas.ps.inquiry.model.PublicInquiryReply;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import com.uas.ps.inquiry.util.FlexJsonUtils;
import com.uas.ps.inquiry.util.HttpUtil;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author dongbw
 * @since 2018年9月26日
 */

public class InquiryForSalerControllerTest extends BaseJunitTest {

    /**
     * localhost
     */
    private static final String localhost = "http://10.1.51.82:24002";

    /**
     * dev
     */
    private static final String dev_url = "http://192.168.253.12:24000";

    /**
     * userUU
     */
    private static final Long userUU = 1000002503L;

    /**
     * enUU
     */
    private static final Long enUU = 10041166L;


    /**
     * saveQuote
     * @throws Exception
     */
    @Test
    public void testSaveQuote() throws Exception {
        PublicInquiry inquiry = new PublicInquiry();
        inquiry.setId(1225L);
        inquiry.setEnUU(enUU);
        inquiry.setRecorderUU(userUU);
        PublicInquiryItem item = new PublicInquiryItem();
        item.setInquiry(inquiry);
        item.setUserName("龙晓兰");
        item.setUserTel("18692990942");
        item.setQutoApp("MALL");
        item.setCurrency("RMB");
        item.setProdTitle("IC");
        item.setStatus((short) 100);
        item.setInbrand("美尔森");
        item.setLeadtime((long) 1);
        item.setVendUserUU(userUU);
        item.setVendUU(10041559L);
        item.setId(139018L);
        item.setSpec("EMC0123SN7406DREMC123EMC");
        item.setIsReplace((short) 0);

        List<PublicInquiryReply> replies = new ArrayList<>();
        PublicInquiryReply reply = new PublicInquiryReply();
        reply.setLapQty((double) 1);
        reply.setPrice((double) 200);
        replies.add(reply);
        item.setReplies(replies);

        String res = HttpUtil.doPost(localhost + "/inquiry/sale/saveQuote",
                FlexJsonUtils.toJsonDeep(item));
        System.out.println(res);
    }

    /**
     * /inquiry/detail
     * @throws Exception
     */
    @Test
    public void testFindById() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 报价明细Id
        params.put("itemId", 1225);
        params.put("en_uu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/sale/inquiry/detail", params);
        System.out.println(JSON.toJSON(res));
    }


    /**
     * /publicInquiry/detail
     * @throws Exception
     */
    @Test
    public void testFindInquiryById() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("id", 1169);
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/sale/publicInquiry/detail", params);
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /attach
     * @throws Exception
     */
    @Test
    public void testAddAttachs() throws Exception {
        Attach attach = new Attach();
        attach.setDate(new Date());
        attach.setName("attach");
        attach.setPath("https://dfs.ubtob.com/group1/M00/88/B7/CgpkyFsZBI-AakOYAAAX5xdZwHw866.jpg");
        String res = HttpUtil.doPost(localhost + "/inquiry/sale/attach", FlexJsonUtils.toJsonDeep(attach));
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /quote
     * @throws Exception
     */
    @Test
    public void testFindBySourceId() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("id", 1169);
        params.put("en_uu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/sale/quote", params);
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /allot/enremind
     * @throws Exception
     */
    @Test
    public void testAllotEnRemind() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("userUU", enUU);
        params.put("enRemindId", 89445);
        String res = HttpUtil.doPost(localhost + "/inquiry/sale/allot/enremind", FlexJsonUtils.toJsonDeep(params));
        System.out.println(res);
    }

    /**
     * /allot/inquiry
     * @throws Exception
     */
    @Test
    public void testAllotInquiry() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("userUU", enUU);
        params.put("itemId", 1169);
        params.put("enUU", enUU);
        String res = HttpUtil.doPost(localhost + "/inquiry/sale/allot/inquiry", FlexJsonUtils.toJsonDeep(params));
        System.out.println(res);
    }

    /**
     * /remind
     * @throws Exception
     */
    @Test
    public void testFindByUserInfo() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("useruu", userUU);
        params.put("enuu", enUU);
        SearchFilter searchFilter = new SearchFilter();
        searchFilter.setEndDate(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2);
        PageInfo pageInfo = new PageInfo(1, 20);
        params.put("pageInfo", pageInfo);
        params.put("searchFilter", searchFilter);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/sale/remind", params);
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /enremind
     * @throws Exception
     */
    @Test
    public void testFindEnRemind() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        // 询价明细Id
        params.put("useruu", userUU);
        params.put("enuu", enUU);
        SearchFilter searchFilter = new SearchFilter();
        searchFilter.setEndDate(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2);
        PageInfo pageInfo = new PageInfo(1, 20);
        params.put("pageInfo", pageInfo);
        params.put("searchFilter", searchFilter);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/sale/enremind", params);
        System.out.println(JSON.toJSON(res));
    }


}
